package models_utils

import (
	"database/sql"
	"errors"
)

type BaseModel struct {
	Name   string
	Status int
	Id     string

	ModelType int
	ImageMode int
	Width     int
	Height    int
	Format    string
}

const (
	FAILED_TRAINING           = -4
	FAILED_PREPARING_TRAINING = -3
	FAILED_PREPARING_ZIP_FILE = -2
	FAILED_PREPARING          = -1

	PREPARING               = 1
	CONFIRM_PRE_TRAINING    = 2
	PREPARING_ZIP_FILE      = 3
	TRAINING                = 4
	READY                   = 5
	READY_ALTERATION        = 6
	READY_ALTERATION_FAILED = -6

	READY_RETRAIN        = 7
	READY_RETRAIN_FAILED = -7
)

type ModelDefinitionStatus int

type LayerType int

const (
	LAYER_INPUT        LayerType = 1
	LAYER_DENSE                  = 2
	LAYER_FLATTEN                = 3
	LAYER_SIMPLE_BLOCK           = 4
)

const (
	MODEL_DEFINITION_STATUS_CANCELD_TRAINING ModelDefinitionStatus = -4
	MODEL_DEFINITION_STATUS_FAILED_TRAINING                        = -3
	MODEL_DEFINITION_STATUS_PRE_INIT                               = 1
	MODEL_DEFINITION_STATUS_INIT                                   = 2
	MODEL_DEFINITION_STATUS_TRAINING                               = 3
	MODEL_DEFINITION_STATUS_PAUSED_TRAINING                        = 6
	MODEL_DEFINITION_STATUS_TRANIED                                = 4
	MODEL_DEFINITION_STATUS_READY                                  = 5
)

type ModelClassStatus int

const (
	MODEL_CLASS_STATUS_TO_TRAIN ModelClassStatus = 1
	MODEL_CLASS_STATUS_TRAINING                  = 2
	MODEL_CLASS_STATUS_TRAINED                   = 3
)

type ModelHeadStatus int

const (
	MODEL_HEAD_STATUS_PRE_INIT ModelHeadStatus = 1
	MODEL_HEAD_STATUS_INIT                     = 2
	MODEL_HEAD_STATUS_TRAINING                 = 3
	MODEL_HEAD_STATUS_TRAINED                  = 4
	MODEL_HEAD_STATUS_READY                    = 5
)

var ModelNotFoundError = errors.New("Model not found error")

func GetBaseModel(db *sql.DB, id string) (base *BaseModel, err error) {
	rows, err := db.Query("select name, status, id, width, height, color_mode, format, model_type from models where id=$1;", id)
	if err != nil {
		return
	}
	defer rows.Close()

	if !rows.Next() {
		return nil, ModelNotFoundError
	}

	base = &BaseModel{}
	var colorMode string
	err = rows.Scan(&base.Name, &base.Status, &base.Id, &base.Width, &base.Height, &colorMode, &base.Format, &base.ModelType)
	if err != nil {
		return nil, err
	}

	base.ImageMode = StringToImageMode(colorMode)
	return
}

func StringToImageMode(colorMode string) int {
	switch colorMode {
	case "greyscale":
		return 1
	case "rgb":
		return 3
	default:
		panic("unkown color mode")
	}
}
